// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pivot Points.
	/// </summary>
	[Description("Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.")]
	public class PivotsPlus : Indicator
	{
		#region Variables
		private	SolidBrush[]		brushes				= {new SolidBrush(Color.Black),new SolidBrush(Color.Black), new SolidBrush(Color.Black),  new SolidBrush(Color.Black),new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),new SolidBrush(Color.Black) };
		private DateTime			cacheSessionBegin	= Cbi.Globals.MinDate;
		private DateTime			cacheSessionEnd		= Cbi.Globals.MinDate;
		private DateTime			cacheSessionDate	= Cbi.Globals.MinDate;
		private DateTime			cacheWeeklyEndDate	= Cbi.Globals.MinDate;
		private DateTime			cacheMonthlyEndDate	= Cbi.Globals.MinDate;
		private	double				currentClose		= 0;
		private DateTime			currentDate			= Cbi.Globals.MinDate;
		private DateTime			currentMonth		= Cbi.Globals.MinDate;
		private DateTime			currentWeek			= Cbi.Globals.MinDate;
		private	double				currentHigh			= double.MinValue;
		private	double				currentLow			= double.MaxValue;
		private Bars				dailyBars			= null;
		private	string				errorData			= "Insufficient historical data to calculate pivots. Increase chart look back period (days back).";
        private float				errorTextWidth		= 0;
        private float				errorTextHeight		= 0;
		private bool				existsHistDailyData	= false;
		private bool				isDailyDataLoaded	= false;
		private ArrayList			newSessionBarIdxArr	= new ArrayList();
		private Data.PivotRange		pivotRangeType		= PivotRange.Daily;
		private HLCCalculationMode	priorDayHLC			= HLCCalculationMode.CalcFromIntradayData;
		public double				pp					= 0;
		public	double				r1					= 0;
		public	double				r2					= 0;
		public	double				r3					= 0;
		public	double				s1					= 0;
		public	double				s2					= 0;
		public	double				s3					= 0;
		private	double				ppub					= 0;   //BC Add
		private	double				pplb					= 0;	//BC Add
		private	double				r1ub					= 0;	//BC Add
		private	double				r1lb					= 0;	//BC Add
		private	double				s1ub					= 0;	//BC Add
		private	double				s1lb					= 0;	//BC Add
		private	double				ydClose					= 0;	//BC Add
		private	double				ydHigh					= 0;	//BC Add
		private	double				ydLow					= 0;	//BC Add
		public	double				fairValue				= 0;	//BC Add
		public	double				settlement				= 0;	//BC Add
			public	double				lowRMS				= 0;	//BC Add
			public	double				highRMS				= 0;	//BC Add
		private	double				tag1					= 0;	//BC Add -Fib
		private	double				tag2					= 0;	//BC Add-Fib
		private	double				tag3					= 0;	//BC Add-Fib
		private	int				cashcloseminute					= 30;	//BC Add-Fib
		private	int				cashclosehour					= 13;	//BC Add-Fib
						
		private DateTime			sessionDateTmp		= Cbi.Globals.MinDate;
		private DateTime			sessionDateDaily	= Cbi.Globals.MinDate;
		private	StringFormat		stringFormatFar		= new StringFormat();
		private	StringFormat		stringFormatNear	= new StringFormat();
		private SolidBrush			textBrush			= new SolidBrush(Color.Black);
		private Font				textFont			= new Font("Arial",30);
		private double				userDefinedClose	= 0;
		private double				userDefinedHigh		= 0;
		private double				userDefinedLow		= 0;
		private int					width				= 10;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Lime,	"PP"));		//BCMod
			Plots[0].Pen.Width = 3;
			Add(new Plot(Color.Yellow,	"R1"));		//BCMod
			Plots[1].Pen.Width = 3;
			Add(new Plot(Color.Yellow,		"S1"));		//BCMod
			Plots[2].Pen.Width = 3;
			Add(new Plot(Color.Red,	"R2"));		//BCMod
			Add(new Plot(Color.Red,		"S2"));		//BCMod
			Add(new Plot(Color.DodgerBlue,	"R3"));		//BCMod
			Add(new Plot(Color.DodgerBlue,	"S3"));	//BCMod
			Add(new Plot(Color.Lime,		"PPUB"));    //BC Add Upper Band
			Add(new Plot(Color.Lime,		"PPLB"));    //BC Add Lower Band
			Add(new Plot(Color.Yellow,		"S1UB"));    //BC Add UB
			Add(new Plot(Color.Yellow,		"S1LB"));    //BC Add LB
			Add(new Plot(Color.Yellow,		"R1UB"));    //BC Add UB
			Add(new Plot(Color.Yellow,		"R1LB"));    //BC Add LB
			//13 items
			Add(new Plot(Color.OrangeRed,		"YDClose"));    //BC Add LB
			Plots[13].Pen.Width = 3;
			Add(new Plot(Color.Orange,		"YDHigh"));    //BC Add LB
			Plots[14].Pen.Width = 3;
			Add(new Plot(Color.Orange,		"YDLow"));    //BC Add LB
			Plots[15].Pen.Width = 3;
			Add(new Plot(Color.SkyBlue,		"TDLow"));    //BC Add LB
			Plots[16].Pen.Width = 3;
			Add(new Plot(Color.SkyBlue,		"TDHigh"));    //BC Add LB
			Plots[17].Pen.Width = 3;
			Add(new Plot(Color.Cyan,		"YDMid"));    //BC Add FairValue
			Plots[18].Pen.Width = 3;
			Add(new Plot(Color.Purple,		"CashSettle "));    //BC Add CashSettle
			Plots[19].Pen.Width = 4;
			Plots[19].Pen.DashStyle = DashStyle.Dash;
			Add(new Plot(Color.Purple,		"LowRMS "));    //BC Add LowRMS
			Plots[20].Pen.Width = 4;
			Plots[20].Pen.DashStyle = DashStyle.Dash;
			Add(new Plot(Color.Purple,		"HighRMS "));    //BC Add HighRMS
			Plots[21].Pen.Width = 4;
			Plots[21].Pen.DashStyle = DashStyle.Dash;
			//22 items
			//Upper & Lower Bands around Pivots
			Plots[6].Pen.DashStyle = DashStyle.Dash;
			Plots[6].Pen.Width = 1;
			Plots[7].Pen.DashStyle = DashStyle.Dash;
			Plots[7].Pen.Width = 1;
			Plots[8].Pen.DashStyle = DashStyle.Dash;
			Plots[8].Pen.Width = 1;
			Plots[9].Pen.DashStyle = DashStyle.Dash;
			Plots[9].Pen.Width = 1;
			Plots[10].Pen.DashStyle = DashStyle.Dash;
			Plots[10].Pen.Width = 1;
			Plots[11].Pen.DashStyle = DashStyle.Dash;
			Plots[11].Pen.Width = 1;
			Plots[12].Pen.DashStyle = DashStyle.Dash;
			Plots[12].Pen.Width = 1;
			
			
			
			
			
			
			
			
			
			
			
			 Add(PeriodType.Day,1);
			
			AutoScale					= false;
			Overlay						= true;
			stringFormatFar.Alignment	= StringAlignment.Far;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			
			if (CurrentBar < 4) return;
			if (Bars == null)
				return; 
			if (!Bars.BarsType.IsIntraday && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType == PivotRange.Daily)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;

			if (!isDailyDataLoaded)
			{
				if (priorDayHLC == HLCCalculationMode.DailyBars && Bars.BarsType.IsIntraday) 
				{
					Enabled = false;
					System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(GetBarsNow));
					return;
				}

				existsHistDailyData = false;
				isDailyDataLoaded	= true;
			}

			IBar dailyBar;
			if (existsHistDailyData) 
			{
				sessionDateDaily = GetLastBarSessionDate(Time[0], Bars, PivotRange.Daily);
				dailyBar = dailyBars.Get(dailyBars.GetBar(sessionDateDaily));

				if (dailyBar.Time.Date > sessionDateDaily.Date)
				{
					for (DateTime i = sessionDateDaily; i >= dailyBars.GetTime(0); i = i.AddDays(-1))
					{
						dailyBar = dailyBars.Get(dailyBars.GetBar(i));
						if (dailyBar.Time.Date == i.Date)
							break;
					}
				}
			} 
			else 
				dailyBar = null;

			double	high		= existsHistDailyData ? dailyBar.High : High[0];
			double	low			= existsHistDailyData ? dailyBar.Low : Low[0];
			double	close		= existsHistDailyData ? dailyBar.Close : Close[0];
			
						
			DateTime lastBarTimeStamp = GetLastBarSessionDate(Time[0], Bars, pivotRangeType);
			if ((currentDate != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Daily && lastBarTimeStamp != currentDate)
				|| (currentWeek != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Weekly && lastBarTimeStamp != currentWeek) 
				|| (currentMonth != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Monthly && lastBarTimeStamp != currentMonth)) 
			{
				pp				= (currentHigh + currentLow + currentClose) / 3;
				s1				= 2 * pp - currentHigh;
				r1				= 2 * pp - currentLow;
				s2				= pp - (currentHigh - currentLow);
				r2				= pp + (currentHigh - currentLow);
				s3				= pp - 2 * (currentHigh - currentLow);
				r3				= pp + 2 * (currentHigh - currentLow);
				ppub		  = pp+((r1-pp)*.1); //BC Add
				pplb		   = pp-((pp-s1)*.1) ;//BC Add
				r1ub		   = r1+((r1-pp)*.1) ;//BC Add
				r1lb			= r1-((r1-pp)*.1) ;//BC Add
				s1ub		   = s1+((r1-pp)*.1); //BC Add
				s1lb		    = s1-((r1-pp)*.1) ;//BC Add
				currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;   //BC Add
				currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;     //BC Add
				currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;      //BC Add
								
				 ydHigh=PriorDayOHLC().PriorHigh[0];	//BC Add
				 ydLow=PriorDayOHLC().PriorLow[0];		//BC Add
				 ydClose=PriorDayOHLC().PriorClose[0];	//BC Add
				fairValue=(ydHigh-ydLow)/2+ydLow;		//BC Add
				highRMS=(ydHigh-ydLow)*.707+ydLow;		//BC Add
				lowRMS=(ydHigh-ydLow)*.293+ydLow;		//BC Add
				//yesterday settlement				
				int barsAgoSettle = CurrentBar - Bars.GetBar(new DateTime(Time[0].Year, Time[0].Month,Time[0].Day,CashCloseHour,CashCloseMinute,0));	
				settlement=Close[barsAgoSettle];
			
			}
			else
			{
				currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
				currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
				currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
			}

			if (pivotRangeType == PivotRange.Daily)
				currentDate = lastBarTimeStamp;
			if (pivotRangeType == PivotRange.Weekly)
				currentWeek = lastBarTimeStamp;
			if (pivotRangeType == PivotRange.Monthly)
				currentMonth = lastBarTimeStamp;

			if ((pivotRangeType == PivotRange.Daily && currentDate != Cbi.Globals.MinDate)
				|| (pivotRangeType == PivotRange.Weekly && currentWeek != Cbi.Globals.MinDate)
				|| (pivotRangeType == PivotRange.Monthly && currentMonth != Cbi.Globals.MinDate))
			{
				PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				PPUB.Set(ppub);  //BC Add
				PPLB.Set(pplb);	//BC Add
				S1UB.Set(r1ub); 	//BC Add
				S1LB.Set(r1lb);	//BC Add
				R1UB.Set(s1ub);	//BC Add
				R1LB.Set(s1lb);	//BC Add
				YDClose.Set(ydClose);	//BC Add
				YDHigh.Set(ydHigh);	//BC Add
				YDLow.Set(ydLow);	//BC Add
				TDLow.Set(currentHigh);	//BC Add
				TDHigh.Set(currentLow);	//BC Add
				FairValue.Set(fairValue);	//BC Add
				Settlement.Set(settlement); //BC Add				
				LowRMS.Set(lowRMS); //BC Add				
				HighRMS.Set(highRMS); //BC Add				
				
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Description("Type of period for pivot points.")]
		[GridCategory("Parameters")]
		public Data.PivotRange PivotRangeType 
		{
			get { return pivotRangeType; }
			set { pivotRangeType = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Description("Approach for calculation the prior day HLC values.")]
		[GridCategory("Parameters")]
		public Data.HLCCalculationMode PriorDayHLC
		{
			get { return priorDayHLC; }
			set { priorDayHLC = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[4]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[6]; }
		}

		/// <summary>		//added PPUB, PPLB, R1UB, R1LB, S1Ub, S1LB, TD High, TDLow, YDHigh, YDLow, YD Close, RMSLevels, CashSettle
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PPUB
		{
			get { return Values[7]; }
		}
		
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PPLB
		{
			get { return Values[8]; }
		}
		
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1UB
		{
			get { return Values[9]; }
		}
		
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1LB
		{
			get { return Values[10]; }
		}
		
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1UB
		{
			get { return Values[11]; }
		}
		
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1LB
		{
			get { return Values[12]; }
		}
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries YDClose
		{
			get { return Values[13]; }
		}
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries YDHigh
		{
			get { return Values[14]; }
		}
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries YDLow
		{
			get { return Values[15]; }
		}
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries TDHigh
		{
			get { return Values[16]; }
		}
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries TDLow
		{
			get { return Values[17]; }
		}
		/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FairValue
		{
			get { return Values[18]; }
		}
			/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Settlement
		{
			get { return Values[19]; }
		}
		
			/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries LowRMS
		{
			get { return Values[20]; }
		}
		
			/// <summary>		//BC Add
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries HighRMS
		{
			get { return Values[21]; }
		}
		
		/// <summary>
		/// close value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session close value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedClose
		{
			get { return userDefinedClose; }
			set { userDefinedClose = value; }
		}

		/// <summary>
		/// high value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session high value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedLow
		{
			get { return userDefinedLow; }
			set { userDefinedHigh = value; }
		}

		/// <summary>
		/// low value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session low value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedHigh
		{
			get { return userDefinedHigh; }
			set { userDefinedLow = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Offset of the pivot lines as # of bars.")]
		[GridCategory("Parameters")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(1, value); }
		}
		
		[Description("CashClose Hour for SettlePrice")]
		[GridCategory("Parameters")]
		public int CashCloseHour
		{
			get { return cashclosehour; }
			set { cashclosehour = Math.Max(1, value); }
		}
		[Description("CashClose Minute for SettlePrice")]
		[GridCategory("Parameters")]
		public int CashCloseMinute
		{
			get { return cashcloseminute; }
			set { cashcloseminute = Math.Max(1, value); }
		}
		#endregion     /BC BC

		#region Miscellaneous

		/// <summary>
		/// Have the .GetBars() call in a seperate thread. Reason: NT internal pool locking would not work if called in main thread.
		/// </summary>
		/// <param name="state"></param>
		private void GetBarsNow(object state)
		{
			if (Disposed)
				return;

			dailyBars			= Data.Bars.GetBars(Bars.Instrument, new Period(PeriodType.Day, 1, Bars.Period.MarketDataType), Bars.From.AddDays(-7), Bars.To, (Session) Bars.Session.Clone(), Data.Bars.SplitAdjust, Data.Bars.DividendAdjust);
			existsHistDailyData	= (dailyBars.Count <= 1) ? false : true;
			isDailyDataLoaded	= true;
			Enabled				= true;

			Cbi.Globals.SynchronizeInvoke.AsyncInvoke(new System.Windows.Forms.MethodInvoker(InvalidateNow), null);
		}

		private DateTime GetLastBarSessionDate(DateTime time, Data.Bars bars, PivotRange pivotRange)
		{
			if (time > cacheSessionEnd)
			{
				if (Bars.BarsType.IsIntraday)
				{
					Bars.Session.GetNextBeginEnd(time, out cacheSessionBegin, out cacheSessionEnd);
					sessionDateTmp = System.TimeZoneInfo.ConvertTime(cacheSessionEnd.AddSeconds(-1), System.TimeZoneInfo.Local, Bars.Session.TimeZoneInfo).Date;
				}
				else
					sessionDateTmp = time.Date;
			}	
			
			if (pivotRange == PivotRange.Daily)
			{	
				if(sessionDateTmp != cacheSessionDate)
				{
					if (newSessionBarIdxArr.Count == 0 || (newSessionBarIdxArr.Count > 0 && CurrentBar > (int) newSessionBarIdxArr[newSessionBarIdxArr.Count - 1]))
						newSessionBarIdxArr.Add(CurrentBar);
					cacheSessionDate = sessionDateTmp;
				}
				return sessionDateTmp;
			}
			else if (pivotRange == PivotRange.Weekly) 
			{
				DateTime tmpWeeklyEndDate = RoundUpTimeToPeriodTime(sessionDateTmp, PivotRange.Weekly);
				if (tmpWeeklyEndDate != cacheWeeklyEndDate)
				{
					if (newSessionBarIdxArr.Count == 0 || (newSessionBarIdxArr.Count > 0 && CurrentBar > (int) newSessionBarIdxArr[newSessionBarIdxArr.Count - 1]))
						newSessionBarIdxArr.Add(CurrentBar);
					cacheWeeklyEndDate = tmpWeeklyEndDate;
				}
				return tmpWeeklyEndDate;
			}
			else // pivotRange == PivotRange.Monthly
			{
				DateTime tmpMonthlyEndDate = RoundUpTimeToPeriodTime(sessionDateTmp, PivotRange.Monthly);
				if (tmpMonthlyEndDate != cacheMonthlyEndDate)
				{
					if (newSessionBarIdxArr.Count == 0 || (newSessionBarIdxArr.Count > 0 && CurrentBar > (int) newSessionBarIdxArr[newSessionBarIdxArr.Count - 1]))
						newSessionBarIdxArr.Add(CurrentBar);
					cacheMonthlyEndDate = tmpMonthlyEndDate;
				}
				return tmpMonthlyEndDate;
			}
		}

		internal void InvalidateNow()
		{
			if (Disposed || ChartControl == null)
				return;

			ChartControl.Invalidate(true);
		}

		/// <summary>
        /// Overload this method to handle the termination of an indicator. Use this method to dispose of any resources vs overloading the Dispose() method.
		/// </summary>
		protected override void OnTermination()
		{
			if (dailyBars != null)
				dailyBars.Dispose();

			textBrush.Dispose();
			foreach (SolidBrush solidBrush in brushes)
				solidBrush.Dispose();
			stringFormatFar.Dispose();	
			stringFormatNear.Dispose();
		}
	
		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			// plot error if data not complete
			if (textBrush.Color != ChartControl.AxisColor || textFont != ChartControl.Font)
			{
				textBrush.Color = ChartControl.AxisColor;
                textFont = ChartControl.Font;

				SizeF errorSize	= graphics.MeasureString(errorData, textFont);
				errorTextWidth	= errorSize.Width + 5;
				errorTextHeight	= errorSize.Height + 5;
			}

			if (priorDayHLC == HLCCalculationMode.CalcFromIntradayData) 
			{
				DateTime lastBarDate = (Bars.Count == 0) ? Cbi.Globals.MaxDate : ChartControl.EquidistantBars ?
										Bars.GetTime(Math.Min(Bars.Count - 1, ChartControl.LastBarPainted)).Date 
										: Bars.GetTime(Math.Min(Bars.Count - 1, Bars.GetBar(ChartControl.LastBarTimePainted))).Date;
				if ((lastBarDate == Cbi.Globals.MaxDate)
					|| (Bars.BarsType.BuiltFrom == PeriodType.Minute
						&& ((pivotRangeType == PivotRange.Monthly && Bars.Count > 0 && Bars.GetTime(0).AddSeconds(-1).Date >= new DateTime(lastBarDate.Year, lastBarDate.Month, 1).AddMonths(-1)) 
							|| (pivotRangeType == PivotRange.Weekly && Bars.Count > 0 && Bars.GetTime(0).AddSeconds(-1).Date >= lastBarDate.AddDays(- (((int) lastBarDate.DayOfWeek) + 1) % 7).AddDays(-7))
							|| (pivotRangeType == PivotRange.Daily && Bars.Count > 0 && Bars.GetTime(0).AddSeconds(-1).Date >= lastBarDate.AddDays(-1))))
					|| (Bars.BarsType.BuiltFrom != PeriodType.Minute
						&& ((pivotRangeType == PivotRange.Monthly && Bars.Count > 0 && Bars.GetTime(0).Date >= new DateTime(lastBarDate.Year, lastBarDate.Month, 1).AddMonths(-1)) 
							|| (pivotRangeType == PivotRange.Weekly && Bars.Count > 0 && Bars.GetTime(0).Date >= lastBarDate.AddDays(-(((int) lastBarDate.DayOfWeek) + 1) % 7).AddDays(-7))
							|| (pivotRangeType == PivotRange.Daily && Bars.Count > 0 && Bars.GetTime(0).Date >= lastBarDate.AddDays(-1)))))
				{
					graphics.DrawString(errorData, ChartControl.Font, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				}
			}
			else if (priorDayHLC == HLCCalculationMode.DailyBars && existsHistDailyData)
			{
				DateTime lastBarDate = (dailyBars.Count == 0) ? Cbi.Globals.MaxDate : ChartControl.EquidistantBars ? 
					dailyBars.GetTime(Math.Min(dailyBars.Count - 1, ChartControl.LastBarPainted)).Date 
					: dailyBars.GetTime(Math.Min(dailyBars.Count - 1, dailyBars.GetBar(ChartControl.LastBarTimePainted))).Date;
				if ((lastBarDate == Cbi.Globals.MaxDate)
					|| (pivotRangeType == PivotRange.Monthly && dailyBars.GetTime(0).Date >= new DateTime(lastBarDate.Year, lastBarDate.Month, 1).AddMonths(-1)) 
					|| (pivotRangeType == PivotRange.Weekly && dailyBars.GetTime(0).Date >= lastBarDate.AddDays(-(((int) lastBarDate.DayOfWeek) + 1) % 7).AddDays(-7))
					|| (pivotRangeType == PivotRange.Daily && dailyBars.GetTime(0).Date >= lastBarDate.AddDays(-1)))
				{
					graphics.DrawString(errorData, ChartControl.Font, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				}
			} 

			float textHeight = ChartControl.Font.GetHeight();
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				int				firstBarIdxToPaint	= -1;
				int				lastX				= -1;
				int				lastY				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries		series				= (DataSeries) Values[seriesCount];

				for (int i = newSessionBarIdxArr.Count - 1; i >= 0; i--)
				{
					int prevSessionBreakIdx = (int) newSessionBarIdxArr[i];
					if (prevSessionBreakIdx <= this.LastBarIndexPainted)
					{
						firstBarIdxToPaint = prevSessionBreakIdx;
						break;
					}
				}

				using (GraphicsPath	path = new GraphicsPath()) 
				{
					if (brush.Color != plot.Pen.Color)	
						brush = new SolidBrush(plot.Pen.Color);

					for (int idx = this.LastBarIndexPainted; idx >= Math.Max(this.FirstBarIndexPainted, this.LastBarIndexPainted - Width); idx--)
					{
						if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
							continue;
						else if (!series.IsValidPlot(idx))
							continue;

						if (idx < firstBarIdxToPaint)
							break;

						double	val = series.Get(idx);
						int		x	= ChartControl.GetXByBarIdx(BarsArray[0], idx);
						int		y	= ChartControl.GetYByValue(this, val);

						if (lastX >= 0)
						{
							if (y != lastY) // Problem here is, that last bar of old day has date of new day
								y = lastY;
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
						lastX	= x;
						lastY	= y;
					}

					graphics.SmoothingMode = SmoothingMode.AntiAlias;
					graphics.DrawPath(plot.Pen, path);
					graphics.SmoothingMode = oldSmoothingMode;
					graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX, lastY - textHeight / 2, stringFormatFar);
				}
			}
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6 - (((int) time.DayOfWeek) + 1) % 7));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PivotsPlus[] cachePivotsPlus = null;

        private static PivotsPlus checkPivotsPlus = new PivotsPlus();

        /// <summary>
        /// Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.
        /// </summary>
        /// <returns></returns>
        public PivotsPlus PivotsPlus(int cashCloseHour, int cashCloseMinute, Data.PivotRange pivotRangeType, Data.HLCCalculationMode priorDayHLC, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return PivotsPlus(Input, cashCloseHour, cashCloseMinute, pivotRangeType, priorDayHLC, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.
        /// </summary>
        /// <returns></returns>
        public PivotsPlus PivotsPlus(Data.IDataSeries input, int cashCloseHour, int cashCloseMinute, Data.PivotRange pivotRangeType, Data.HLCCalculationMode priorDayHLC, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            if (cachePivotsPlus != null)
                for (int idx = 0; idx < cachePivotsPlus.Length; idx++)
                    if (cachePivotsPlus[idx].CashCloseHour == cashCloseHour && cachePivotsPlus[idx].CashCloseMinute == cashCloseMinute && cachePivotsPlus[idx].PivotRangeType == pivotRangeType && cachePivotsPlus[idx].PriorDayHLC == priorDayHLC && Math.Abs(cachePivotsPlus[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cachePivotsPlus[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cachePivotsPlus[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cachePivotsPlus[idx].Width == width && cachePivotsPlus[idx].EqualsInput(input))
                        return cachePivotsPlus[idx];

            lock (checkPivotsPlus)
            {
                checkPivotsPlus.CashCloseHour = cashCloseHour;
                cashCloseHour = checkPivotsPlus.CashCloseHour;
                checkPivotsPlus.CashCloseMinute = cashCloseMinute;
                cashCloseMinute = checkPivotsPlus.CashCloseMinute;
                checkPivotsPlus.PivotRangeType = pivotRangeType;
                pivotRangeType = checkPivotsPlus.PivotRangeType;
                checkPivotsPlus.PriorDayHLC = priorDayHLC;
                priorDayHLC = checkPivotsPlus.PriorDayHLC;
                checkPivotsPlus.UserDefinedClose = userDefinedClose;
                userDefinedClose = checkPivotsPlus.UserDefinedClose;
                checkPivotsPlus.UserDefinedHigh = userDefinedHigh;
                userDefinedHigh = checkPivotsPlus.UserDefinedHigh;
                checkPivotsPlus.UserDefinedLow = userDefinedLow;
                userDefinedLow = checkPivotsPlus.UserDefinedLow;
                checkPivotsPlus.Width = width;
                width = checkPivotsPlus.Width;

                if (cachePivotsPlus != null)
                    for (int idx = 0; idx < cachePivotsPlus.Length; idx++)
                        if (cachePivotsPlus[idx].CashCloseHour == cashCloseHour && cachePivotsPlus[idx].CashCloseMinute == cashCloseMinute && cachePivotsPlus[idx].PivotRangeType == pivotRangeType && cachePivotsPlus[idx].PriorDayHLC == priorDayHLC && Math.Abs(cachePivotsPlus[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cachePivotsPlus[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cachePivotsPlus[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cachePivotsPlus[idx].Width == width && cachePivotsPlus[idx].EqualsInput(input))
                            return cachePivotsPlus[idx];

                PivotsPlus indicator = new PivotsPlus();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.CashCloseHour = cashCloseHour;
                indicator.CashCloseMinute = cashCloseMinute;
                indicator.PivotRangeType = pivotRangeType;
                indicator.PriorDayHLC = priorDayHLC;
                indicator.UserDefinedClose = userDefinedClose;
                indicator.UserDefinedHigh = userDefinedHigh;
                indicator.UserDefinedLow = userDefinedLow;
                indicator.Width = width;
                Indicators.Add(indicator);
                indicator.SetUp();

                PivotsPlus[] tmp = new PivotsPlus[cachePivotsPlus == null ? 1 : cachePivotsPlus.Length + 1];
                if (cachePivotsPlus != null)
                    cachePivotsPlus.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachePivotsPlus = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PivotsPlus PivotsPlus(int cashCloseHour, int cashCloseMinute, Data.PivotRange pivotRangeType, Data.HLCCalculationMode priorDayHLC, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.PivotsPlus(Input, cashCloseHour, cashCloseMinute, pivotRangeType, priorDayHLC, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.
        /// </summary>
        /// <returns></returns>
        public Indicator.PivotsPlus PivotsPlus(Data.IDataSeries input, int cashCloseHour, int cashCloseMinute, Data.PivotRange pivotRangeType, Data.HLCCalculationMode priorDayHLC, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.PivotsPlus(input, cashCloseHour, cashCloseMinute, pivotRangeType, priorDayHLC, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PivotsPlus PivotsPlus(int cashCloseHour, int cashCloseMinute, Data.PivotRange pivotRangeType, Data.HLCCalculationMode priorDayHLC, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.PivotsPlus(Input, cashCloseHour, cashCloseMinute, pivotRangeType, priorDayHLC, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Intraday Pivot Points w PP Bands and other levels of interest from Yest I-Day range.
        /// </summary>
        /// <returns></returns>
        public Indicator.PivotsPlus PivotsPlus(Data.IDataSeries input, int cashCloseHour, int cashCloseMinute, Data.PivotRange pivotRangeType, Data.HLCCalculationMode priorDayHLC, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PivotsPlus(input, cashCloseHour, cashCloseMinute, pivotRangeType, priorDayHLC, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }
    }
}
#endregion
